/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2003      by Advanced Data Controls, Corp
 *  Copyright (C) 2004 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2006 by GJ Business Division RICOH COMPANY,LTD. JAPAN
 *  Copyright (C) 2007-     by Monami Software Limited Partnership, JAPAN
 *  Copyright (C) 2009      by GJ Designing Center Print Business Group
 *                             RICOH COMPANY,LTD. JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: sys_config.c,v 1.1 2006/06/11 11:18:56 roi Exp $
 */

/*
 *    åȥƥ¸⥸塼mindstormsnxtѡ
 */

#include "jsp_kernel.h"
#include <mindstormsnxt.h>


/*interrupt mask*/
UW int_mode_table[MAX_INT_NUM]={
    0,
    SMR_SYSIRQ,
    SMR_PIOA,
    0,
    0,
    SMR_SPI,
    SMR_US0,
    SMR_US1,
    SMR_SSC,
    SMR_TWI,
    SMR_PMW,
    0,
    SMR_TC0,
    SMR_TC1,
    SMR_TC2,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0
};


/* Default handlers for the three general kinds of interrupts that the
 * ARM core has to handle. These are defined in irq.s, and just freeze
 * the board in an infinite loop.
 */
extern void default_isr(void);
extern void default_fiq(void);
extern void spurious_isr(void);
extern void nxt_toppers_stop(void);

/*
 *  åȥƥ¸ν
 */
void
sys_initialize()
{
	int i;

   /* If we're coming from a warm boot, the AIC may be in a weird
    * state. Do some cleaning up to bring the AIC back into a known
    * state:
    *  - All interrupt lines disabled,
    *  - No interrupt lines handled by the FIQ handler,
    *  - No pending interrupts,
    *  - AIC idle, not handling an interrupt.
    */
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_IDCR), 0xFFFFFFFF);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_FFDR), 0xFFFFFFFF);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_ICCR), 0xFFFFFFFF);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_EOICR), 1);

   /* Enable debug protection. This is necessary for JTAG debugging, so
    * that the hardware debugger can read AIC registers without
    * triggering side-effects.
    */
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_DCR), 1);

   /* Set default handlers for all interrupt lines. */
	for (i = 0; i < 32; i++) {
		sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SMR+i*4), 0);
		sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+i*4), (unsigned long)default_isr);
	}
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+IRQ_FIQ_PID*4), (unsigned long)default_fiq);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+TOFF_AIC_SPU), (unsigned long)spurious_isr);
}


/*
 *  åȥƥνλ
 */
void
sys_exit(void)
{
    nxt_toppers_stop();
    while(1);
}


/*
 *  åȥƥʸ
 */

void
sys_putc(char c)
{
}


/*
 *  ߥϥɥ
 *
 *  ֹ inhno γߥϥɥεưϤ inthdr ꤹ롥
 */
void
define_inh(INHNO inhno, FP inthdr)
{
    assert(inhno < MAX_INT_NUM);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_IDCR), (1 << inhno));
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SMR+inhno*4), int_mode_table[inhno]);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+inhno*4), (VW)inthdr);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_IECR), (1 << inhno));
}

/*
 *  ̤γߤäν
 */
void
undef_interrupt(){
    syslog(LOG_EMERG, "Unregistered Interrupt occurs.");
    while(1);
}

