/*
 *  TOPPERS Automotive Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *      Automotive Kernel
 * 
 *  Copyright (C) 2004-2006 by Witz Corporation, JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 */

/*
 *	ץåȰ¸ꥢIO⥸塼(mindstorms-nxt)
 */

/*****************************************************************************
 * FILE: ecrobot_bluetooth.c
 *
 * COPYRIGHT 2008 Takashi Chikamasa <takashic@cybernet.co.jp>
 *
 * <About leJOS NXJ>
 *  leJOS NXJ is a full firmware replacement of LEGO Mindstorms NXT and 
 *  designed for Java programming environment for the NXT 
 *  ( For more detailed information, please see: http://lejos.sourceforge.net/ )
 *  In the leJOS NXJ distribution, C source files for NXT platform layer is also
 *  included besides with the Java VM. The platform C source code is well
 *  structured, comprehensive, and achieved higher performance than the LEGO's
 *  one. Therefore, leJOS NXJ (platform) is also the best GCC based C/C++  
 *  development platform for NXT.
 *
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is TinyVM code, first released March 6, 2000,
 *  later released as leJOS on September 23, 2000.
 *
 *  The Initial Developer of the Original Code is Jose H. Solorzano.
 *
 *  Contributor(s): see leJOS NXJ ACKNOWLEDGEMENTS .
 *
 * <About TOPPERS OSEK>
 *  TOPPERS OSEK is an open source OSEK kernel and developed by TOPPERS project.
 *  TOPPERS(Toyohashi OPen Platform for Embedded Real-time Systems) has been managed 
 *  by a Non Profit Organization founded in Sep. 2003 and has been led by Professor
 *  Hiroaki Takada of Nagoya University in Japan. 
 *
 *  TOPPERS OSEK program is covered by the TOPPERS License as published
 *  by the TOPPERS PROJECT (http://www.toppers.jp/en/index.html).
 *
 *****************************************************************************/

#include <mindstormsnxt.h>
#include "hw_serial.h"

#define BT_BUF_SIZE 254 /* maximum size of Bluetooth Tx/Rx buffer in byte */

#define MSG_ACCEPT_CONNECTION 9
#define MSG_PIN_CODE 10
#define MSG_OPEN_STREAM 11
#define MSG_CONNECT_RESULT 19
#define MSG_REQUEST_PIN_CODE 21
#define MSG_REQUEST_CONNECTION 22

#define MSG_GET_LOCAL_ADDR 39
#define MSG_GET_LOCAL_ADDR_RESULT 40

typedef void * VP;

INT32 connect_state;		/* BLUETOOTH ³ */

static UINT8 receiveBuf[BT_BUF_SIZE];
static volatile UINT8 handle;
static BTCB  *pbtcb;
static BTPACKET *ppacket;
static INT32    plen;

UINT8 bluetooth_device_address[8];

/**
 * send Bluetooth command
 *
 * @param cmd: Bluetooth command
 * @param buflen: length of the command to send
 */
static void send_bt_command(BTCB *btcb, UINT8 *cmd, INT32 bufLen)
{
	BTPACKET *p;
	UINT8    *buf;
    INT32    len;
	int i;
	int checkSum = 0;

	bluetooth_get_packet(btcb, &p, &len);
	buf = &p->buf[0];
	buf[0] = (UINT8) (bufLen + 2);

	for(i = 0; i < bufLen; i++){
		buf[i+1] = cmd[i];
		checkSum += cmd[i];
	}

    checkSum = -checkSum;
    buf[bufLen+2] = (UINT8)((checkSum >> 8) & 0xff);
    buf[bufLen+3] = (UINT8)(checkSum & 0xff);
	p->len = bufLen+3;

	bluetooth_send(btcb, p);
}

/**
 * read Bluetooth command
 *
 * @param cmd: Bluetooth command
 * @param buflen: length of the command buffer
 * @return length of the return command
 */
static UINT32 read_bt_command(BTCB *btcb, UINT8 *cmd, UINT32 bufLen)
{
	INT32  checkSum, negSum, i;
	UINT32 len;

	memset(&receiveBuf[0],0,sizeof(receiveBuf));
	bluetooth_receive(btcb, &receiveBuf[0]);
	len = receiveBuf[0];
	cmd[0] = (UINT8) len;

	if (len == 0)
		return 0;

	checkSum = len;

	if (len-1 <= bufLen){
		for(i = 1; i < len-1; i++){
			cmd[i] = receiveBuf[i];
			checkSum += (cmd[i] & 0xff);
		}
		negSum = (receiveBuf[len-1] & 0xff) << 8;
		negSum += (receiveBuf[len] & 0xff);
		if (checkSum + negSum == 65536){
			return len-1;
		}
		else{
			return 0;
		}
	}
	return 0;
}

/*
 *  BLUETOOTH³֤Ф
 */
INT32 bluetooth_get_connect_state(INT32 id)
{
	return connect_state;
}

/*
 *  BLUETOOTH³֤ѹ
 */
void bluetooth_set_connect_state(INT32 id, INT32 state)
{
    connect_state = state;
}

/**
 * get Bluetooth local device address
 *
 * @param bd_addr: buffer for getting Bluetooth Device Address
 *                 (7bytes hex array data)
 * @return: 1(succeeded)/0(failed)
 */
UINT8 get_bluetooth_device_address(BTCB *btcb, UINT8* bd_addr)
{
	int   i, count = 0;
	UINT8 msg[1];
	UINT8 reply[32];

	msg[0] = MSG_GET_LOCAL_ADDR;
	send_bt_command(btcb, msg, 1);

	while(count < 3000){
		read_bt_command(btcb, reply, 32);

		if (reply[0] != 0 && reply[1] == MSG_GET_LOCAL_ADDR_RESULT)
		{
			for(i=0;i<7;i++)
			{
				bd_addr[i] = reply[i+2];
			}
			return 1;
		}
		sil_dly_nse(10*1000000);
		count += 10;
	}
	return 0;
}

/**
 * open a Bluetooth stream with the paired device
 */
static void open_bluetooth_stream(BTCB *btcb)
{
	UINT8 msg[2];

	if (connect_state == BLUETOOTH_CONNECTED){
		msg[0] = MSG_OPEN_STREAM;
		msg[1] = handle;
		send_bt_command(btcb, msg, 2);
		sil_dly_nse(100*1000000);

		bluetooth_set_arm7_cmd(); /* set stream mode */
		connect_state = BLUETOOTH_STREAM;
		ppacket = 0;
	}
}

/**
 * connect the slave device to a master device
 * NOTE that this API is assumed to be invoked in a loop to establish a connection
 *
 * @param pin: pin code (8 to 16 characters)
 * @return: Bluetooth device status (BLUETOOTH_NO_INIT/BLUETOOTH_INITIALIZED/BLUETOOTH_CONNECTED/BLUETOOTH_STREAM)
 */
static INT32 connect_bluetooth_slave(const INT8 *pin)
{
	int   i;
	UINT8 msg[32];
	UINT8 reply[32];
	UINT8 dummy[32];
	UINT8 bd_addr[7];

	if (connect_state == BLUETOOTH_DEVICE_INITIALIZED){
		if((pbtcb = blutooth_open_device(0)) != 0){
			connect_state = BLUETOOTH_NO_INIT;
		}
		else{
			connect_state = BLUETOOTH_CONNECT_ERROR;
		}
	}
	else if (connect_state == BLUETOOTH_NO_INIT){
		if (get_bluetooth_device_address(pbtcb, bluetooth_device_address)){
			connect_state = BLUETOOTH_INITIALIZED;
		}
		else{
			connect_state = BLUETOOTH_CONNECT_ERROR;
		}
	}
	else if (connect_state == BLUETOOTH_INITIALIZED){
		/* wait for a master device
		 * (based on LEJOS Bluetooth.waitForConnection)
    	 */
		read_bt_command(pbtcb, reply, 32);

		if (reply[0] != 0){
			if (reply[1] == MSG_REQUEST_PIN_CODE){
				for(i = 0; i < 7; i++){
					bd_addr[i] = reply[i+2];
				}

				memset(msg,0,sizeof(msg));
				msg[0] = MSG_PIN_CODE;
				/* fill in slave bd_addr */
				for(i = 0; i < 7; i++){
					msg[i+1] = bd_addr[i];
				}
				/* fill in pin code */
				for(i = 0; i<16 && pin[i] != '\0'; i++){
					msg[i+8] = (UINT8)pin[i];
				}
				send_bt_command(pbtcb, msg, 24);
			}
			else if (reply[1] == MSG_REQUEST_CONNECTION){
				for(i = 0; i < 7; i++){
					bd_addr[i] = reply[i+2];
				}
				msg[0] = MSG_ACCEPT_CONNECTION;
				msg[1] = 1;
				send_bt_command(pbtcb, msg, 2);
			}
			else if (reply[1] == MSG_CONNECT_RESULT){
				sil_dly_nse(300*1000000);
				read_bt_command(pbtcb, dummy, 32);

				if (dummy[0] == 0){
					handle = reply[3];
					connect_state = BLUETOOTH_CONNECTED;
				}
			}
		}
	}
	return connect_state;
}

/**
 * initialize NXT as a Bluetooth slave device
 * NOTE that this API is assumed to be invoked in a loop to establish a connection
 *
 * @param pin: pin code (8 to 16 characters)
 */
void init_bluetooth_slave(const INT8 *pin)
{
	connect_bluetooth_slave(pin);
	open_bluetooth_stream(pbtcb);
}

UINT8 *bluetooth_get_device_address(void)
{
	return bluetooth_device_address;
}

/*
 *  ꥢIO⥸塼
 *
 *  ߶ػ߾֤ǸƽФȡ
 */
void
InitHwSerial( void )
{
	bluetooth_init();
}	/* InitHwSerial	*/

/*
 *  ꥢIO⥸塼߽
 *
 *  ߶ػ߾֤ǸƽФȡ
 *  ̤ǽϤλƤ뤳Ȥݾ㤹뤳ȡ
 */
void
TermHwSerial( void )
{
	FlushSerial();
}	/* TermHwSerial	*/

/*
 *  ƤӻؤƥȤǤʸ
 *
 *  ܴؿϳ߶ػ߾֤ǸƤӽФȡ
 *  ԥɤʤɤ楳ɤ׵¦ǹԤȡ
 */
void
PutHwChar( UINT8 character )
{
	UINT8 *buf;

	if(connect_state == BLUETOOTH_STREAM){
		if(ppacket == 0){
			buf = 0;
			bluetooth_get_packet(pbtcb, &ppacket, &plen);
			if(ppacket != 0){
				ppacket->len = 0;
				buf = &ppacket->buf[0];
			}
		}
		else
			buf = &ppacket->buf[ppacket->len];
		if(buf != 0){
			*buf = character;
			ppacket->len++;
			if(ppacket->len >= plen){
				bluetooth_send(pbtcb, ppacket);
				ppacket = 0;
			}
		}
	}
}	/* PutHwChar	*/

/*
 *  ǧؿ
 */
void
RxHwSerialCheck(void)
{
	static int bcount = 0;

	if(++bcount > 50){
		if(connect_state == BLUETOOTH_STREAM && bluetooth_received_count(pbtcb) > 0){
			RxSerialInt(bluetooth_recvchar(pbtcb));	/* Хå¹ */
		}
		bcount = 0;
	}
}

/*
 *  BLUETOOTHѥåȤΥեå
 */
void
FlushSerial(void)
{
	if(connect_state == BLUETOOTH_STREAM && ppacket != 0){
		bluetooth_send(pbtcb, ppacket);
		ppacket = 0;
	}
}


